@@ -23,8 +23,13 @@ let makeIntervals d =
 
 let g = [1;3;5;9;12;13;14]
 
-let tests = [ "fsharp", makeIntervals 2, g;
-              "ocaml", make_intervals 2, g;
+let repeat f n =
+  for _i = 1 to n do
+    ignore (f g)
+  done
+
+let tests = [ "fsharp", repeat (makeIntervals 2);
+              "ocaml", repeat (make_intervals 2);
             ]
 
-let () = Bench.bench tests
+let () = Bench.bench_n tests |> Bench.run_outputs
