@@ -21,6 +21,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
 
+##V>=5##module Pervasives := Stdlib
+
 (** Additional and modified functions for lists.
 
     The OCaml standard library provides a module for list functions.
@@ -57,7 +59,9 @@
     @author David Teller
 *)
 
-type 'a t = 'a list
+##V<4.08##type 'a t = 'a list
+##V>=4.08##type 'a t = 'a list = [] | (::) of 'a * 'a list
+
 (**The type of lists*)
 
 include BatEnum.Enumerable with type 'a enumerable = 'a t
@@ -71,30 +75,51 @@ val is_empty : 'a list -> bool
 (** [is_empty e] returns true if [e] does not contains any element. *)
 
 val cons : 'a -> 'a list -> 'a list
-(** [cons h t] returns the list starting with [h] and continuing as [t] *)
-
-val first : 'a list -> 'a
-(** Returns the first element of the list, or @raise Empty_list if
-    the list is empty (similar to [hd]). *)
+(** [cons h t] returns the list starting with [h] and continuing as [t]. *)
 
 val hd : 'a list -> 'a
-(** Similar to [first], but @raise Failure if the list is empty. *)
+(** Returns the first element of the list, or @raise Failure if
+    the list is empty. *)
+
+
+val first : 'a list -> 'a
+(** Alias to hd *)
 
 val tl : 'a list -> 'a list
 (** Return the given list without its first element.
     @raise Failure if the list is empty. *)
 
 val last : 'a list -> 'a
-(** Returns the last element of the list, or @raise Empty_list if
+(** Returns the last element of the list, or @raise Invalid_argument if
     the list is empty. This function takes linear time. *)
 
 val length : 'a list -> int
 (** Return the length (number of elements) of the given list. *)
 
+val compare_lengths : 'a list -> 'b list -> int
+(** Compare the lengths of two lists. [compare_lengths l1 l2] is
+   equivalent to [compare (length l1) (length l2)], except that
+   the computation stops after itering on the shortest list.
+   @since 2.7.0
+ *)
+
+val compare_length_with : 'a list -> int -> int
+(** Compare the length of a list to an integer. [compare_length_with l n] is
+   equivalent to [compare (length l) n], except that
+   the computation stops after at most [n] iterations on the list.
+   @since 2.7.0
+*)
+
 val at : 'a list -> int -> 'a
 (** [at l n] returns the n-th element of the list [l] or
     @raise Invalid_argument if the index is outside of [l] bounds.  O(l) *)
 
+val at_opt : 'a list -> int -> 'a option
+(** [at_opt] returns the n-th element of the list [l] or None if the index is
+    beyond the length of [l].
+    @since 2.7.0
+    @raise Invalid_argument if the index is negative *)
+
 val rev : 'a list -> 'a list
 (** List reversal. *)
 
@@ -111,8 +136,9 @@ val shuffle : ?state:Random.State.t -> 'a list -> 'a list
  *)
 
 val append : 'a list -> 'a list -> 'a list
-(** Catenate two lists.  Same function as the infix operator [@].
-    Tail-recursive O(length of the first argument).*)
+(** [append l1 l2] is a concatenation of [l1] and [l2].
+    Same function as the infix operator [@].
+    Tail-recursive. This function takes O([length l1]) time. *)
 
 val rev_append : 'a list -> 'a list -> 'a list
 (** [List.rev_append l1 l2] reverses [l1] and concatenates it to [l2]. *)
@@ -120,8 +146,7 @@ val rev_append : 'a list -> 'a list -> 'a list
 val concat : 'a list list -> 'a list
 (** Concatenate a list of lists.  The elements of the argument are all
     concatenated together (in the same order) to give the result.
-    Tail-recursive
-    (length of the argument + length of the longest sub-list). *)
+    Tail-recursive. *)
 
 val flatten : 'a list list -> 'a list
 (** Same as [concat]. *)
@@ -178,17 +203,38 @@ val unfold: 'b -> ('b -> ('a * 'b) option) -> 'a list
     @since 2.1
 *)
 
-val unfold_exc : (unit -> 'a) -> 'a list * exn
+val unfold_exn : (unit -> 'a) -> 'a list * exn
 (** Creates a list containing the results of sequential calls
     to [f()]. [f()] is called repeatedly until it throws an exception.
     Both the results list, as well as the exception
     thrown are returned in a [(results_list, exn)] pair.
-    Warning: if calls to [f()] never throw an exception, unfold_exc
+    Warning: if calls to [f()] never throw an exception, unfold_exn
     is an infinite loop.
 
-    @since 2.3.0
+    @since 3.2.0
 *)
 
+val unfold_exc : (unit -> 'a) -> 'a list * exn
+(** Alias for [unfold_exn].
+    @deprecated use {!unfold_exn}
+    @since 2.3.0 *)
+
+##V>=4.12##(**{6 Comparison}*)
+##V>=4.12##
+##V>=4.12##val equal : ('a -> 'a -> bool) -> 'a list -> 'a list -> bool
+##V>=4.12##(** [equal eq [a1; ...; an] [b1; ..; bm]] holds when
+##V>=4.12##    the two input lists have the same length, and for each
+##V>=4.12##    pair of elements [ai], [bi] at the same position we have
+##V>=4.12##    [eq ai bi].
+##V>=4.12##
+##V>=4.12##    Note: the [eq] function may be called even if the
+##V>=4.12##    lists have different length. If you know your equality
+##V>=4.12##    function is costly, you may want to check {!compare_lengths}
+##V>=4.12##    first.
+##V>=4.12##
+##V>=4.12##    @since 3.3.0 and 4.12.0
+##V>=4.12##*)
+
 (**{6 Iterators}*)
 
 val iter : ('a -> unit) -> 'a list -> unit
@@ -221,6 +267,9 @@ val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b list -> 'a
 (** [List.fold_left f a [b0; b1; ...; bn]] is
     [f (... (f (f a b0) b1) ...) bn]. *)
 
+val fold : ('a -> 'b -> 'a) -> 'a -> 'b list -> 'a
+(** Alias for [fold_left]. *)
+
 val fold_lefti : ('a -> int -> 'b -> 'a) -> 'a -> 'b list -> 'a
 (** As [fold_left], but with the index of the element, from [0] to
     [length li - 1], as additional argument.
@@ -261,22 +310,30 @@ val fold_left_map : ('a -> 'b -> 'a * 'c) -> 'a -> 'b list -> 'a * 'c list
     @since 2.6.0
 *)
 
-val max : 'a list -> 'a
+val max : ?cmp:('a -> 'a -> int) -> 'a list -> 'a
 (** [max l] returns the largest value in [l] as judged by
-    [Pervasives.compare] *)
+    [Pervasives.compare] (by default). You can provide another
+    comparison function via the optional [cmp] parameter.
+    @raise Invalid_argument on an empty list.
+*)
 
-val min : 'a list -> 'a
+val min : ?cmp:('a -> 'a -> int) -> 'a list -> 'a
 (** [min l] returns the smallest value in [l] as judged by
-    [Pervasives.compare] *)
+    [Pervasives.compare] (by default). You can provide another
+    comparison function via the optional [cmp] parameter.
+    @raise Invalid_argument on an empty list.
+*)
 
 val sum : int list -> int
-(** [sum l] returns the sum of the integers of [l]
-    @raise Invalid_argument on the empty list.
+(** [sum l] returns the sum of the integers of [l].
+    Returns [0] on the empty list.
+    Note: prior to 2.11.0, used to raise Invalid_argument on the empty list.
  *)
 
 val fsum : float list -> float
-(** [fsum l] returns the sum of the floats of [l]
-    @raise Invalid_argument on the empty list.
+(** [fsum l] returns the sum of the floats of [l].
+    Returns [0.] on the empty list.
+    Note: prior to 2.11.0, used to raise Invalid_argument on the empty list.
  *)
 
 val favg : float list -> float
@@ -299,7 +356,15 @@ val min_max : ?cmp:('a -> 'a -> int) -> 'a list -> 'a * 'a
     @raise Invalid_argument on an empty list.
 
     @since 2.1
- *)
+*)
+
+##V>=4.07##val to_seq : 'a list -> 'a Seq.t
+##V>=4.07##(** Iterate on the list
+##V>=4.07##    @since 2.10.0 and OCaml 4.07 *)
+
+##V>=4.07##val of_seq : 'a Seq.t -> 'a list
+##V>=4.07##(** Create a list from the iterator
+##V>=4.07##    @since 2.10.0 and OCaml 4.07 *)
 
 (** {6 Iterators on two lists} *)
 
@@ -307,26 +372,24 @@ val min_max : ?cmp:('a -> 'a -> int) -> 'a list -> 'a * 'a
 val iter2 : ('a -> 'b -> unit) -> 'a list -> 'b list -> unit
 (** [List.iter2 f [a0; a1; ...; an] [b0; b1; ...; bn]] calls in turn
     [f a0 b0; f a1 b1; ...; f an bn].
-    @raise Different_list_size if the two lists have
-    different lengths. *)
+    @raise Invalid_argument if two lists have different lengths. *)
 
 val iter2i : (int -> 'a -> 'b -> unit) -> 'a list -> 'b list -> unit
 (** [List.iter2i f [a0; a1; ...; an] [b0; b1; ...; bn]] calls in turn
     [f 0 a0 b0; f 1 a1 b1; ...; f n an bn].
-    @raise Different_list_size or Invalid_argument if the two lists
-    have different lengths. *)
+    @raise Invalid_argument if two lists have different lengths. *)
 
 val map2 : ('a -> 'b -> 'c) -> 'a list -> 'b list -> 'c list
 (** [List.map2 f [a0; a1; ...; an] [b0; b1; ...; bn]] is
     [[f a0 b0; f a1 b1; ...; f an bn]].
-    @raise Different_list_size if the two lists have
-    different lengths.  Tail-recursive. *)
+    @raise Invalid_argument if two lists have different lengths.
+    Tail-recursive. *)
 
 val map2i : (int -> 'a -> 'b -> 'c) -> 'a list -> 'b list -> 'c list
 (** [List.map2i f [a0; a1; ...; an] [b0; b1; ...; bn]] is
     [[f 0 a0 b0; f 1 a1 b1; ...; f n an bn]].
-    @raise Different_list_size or Invalid_argument if the two lists
-    have different lengths. Tail-recursive. *)
+    @raise Invalid_argument if two lists have different lengths.
+    Tail-recursive. *)
 
 val rev_map2 : ('a -> 'b -> 'c) -> 'a list -> 'b list -> 'c list
 (** [List.rev_map2 f l1 l2] gives the same result as
@@ -336,15 +399,13 @@ val rev_map2 : ('a -> 'b -> 'c) -> 'a list -> 'b list -> 'c list
 val fold_left2 : ('a -> 'b -> 'c -> 'a) -> 'a -> 'b list -> 'c list -> 'a
 (** [List.fold_left2 f a [b0; b1; ...; bn] [c0; c1; ...; cn]] is
     [f (... (f (f a b0 c0) b1 c1) ...) bn cn].
-    @raise Different_list_size if the two lists have
-    different lengths. *)
+    @raise Invalid_argument if two lists have different lengths. *)
 
 val fold_right2 : ('a -> 'b -> 'c -> 'c) -> 'a list -> 'b list -> 'c -> 'c
 (** [List.fold_right2 f [a0; a1; ...; an] [b0; b1; ...; bn] c] is
     [f a0 b0 (f a1 b1 (... (f an bn c) ...))].
-
-    @raise Different_list_size if the two lists have
-    different lengths.  Tail-recursive. *)
+    @raise Invalid_argument if two lists have different lengths.
+    Tail-recursive. *)
 
 
 (**{6 List scanning}*)
@@ -354,7 +415,7 @@ val mem : 'a -> 'a list -> bool
     to an element of [l]. *)
 
 val mem_cmp : ('a -> 'a -> int) -> 'a -> 'a list -> bool
-(** Same as {!List.mem}, but the comparator function is explicitely
+(** Same as {!List.mem}, but the comparator function is explicitly
     provided.
     @since 2.2.0 *)
 
@@ -379,14 +440,12 @@ val exists : ('a -> bool) -> 'a list -> bool
 val for_all2 : ('a -> 'b -> bool) -> 'a list -> 'b list -> bool
 (** Same as {!List.for_all}, but for a two-argument predicate.
 
-    @raise Invalid_argument if the two lists have
-    different lengths. *)
+    @raise Invalid_argument if two lists have different lengths. *)
 
 val exists2 : ('a -> 'b -> bool) -> 'a list -> 'b list -> bool
 (** Same as {!List.exists}, but for a two-argument predicate.
 
-    @raise Invalid_argument if the two lists have
-    different lengths. *)
+    @raise Invalid_argument if two lists have different lengths. *)
 
 val subset : ('a -> 'b -> int) -> 'a list -> 'b list -> bool
 (** [subset cmp l l'] check if all elements of the list [l]
@@ -404,12 +463,26 @@ val find : ('a -> bool) -> 'a list -> 'a
     @raise Not_found if there is no value that satisfies [p] in the
     list [l]. *)
 
+val find_opt: ('a -> bool) -> 'a list -> 'a option
+(** [find_opt p l] returns the first element of the list [l] that
+    satisfies the predicate [p], or [None] if there is no value that
+    satisfies [p] in the list [l].
+    @since 2.7.0 *)
+
+##V>=4.10##val find_map_opt: ('a -> 'b option) -> 'a list -> 'b option
+##V>=4.10##(** [find_map_opt f l] applies [f] to the elements of [l] in order,
+##V>=4.10##    and returns the first result of the form [Some v], or [None]
+##V>=4.10##    if none exist.
+##V>=4.10##
+##V>=4.10##    @since 2.12.0 and OCaml 4.10
+##V>=4.10##*)
+
 val find_exn : ('a -> bool) -> exn -> 'a list -> 'a
 (** [find_exn p e l] returns the first element of [l] such as [p x]
     returns [true] or raises [e] if such an element has not been found. *)
 
 val findi : (int -> 'a -> bool) -> 'a list -> (int * 'a)
-(** [findi p e l] returns the first element [ai] of [l] along with its
+(** [findi p l] returns the first element [ai] of [l] along with its
     index [i] such that [p i ai] is true, or @raise Not_found if no
     such element has been found. *)
 
@@ -419,6 +492,12 @@ val find_map : ('a -> 'b option) -> 'a list -> 'b
     once found or @raise Not_found if no element matches the
     predicate.  See also {!filter_map}. *)
 
+##V>=4.10##val concat_map : ('a -> 'b list) -> 'a list -> 'b list
+##V>=4.10##(** [List.concat_map f l] gives the same result as
+##V>=4.10##    {!List.concat}[ (]{!List.map}[ f l)]. Tail-recursive.
+##V>=4.10##
+##V>=4.10##    @since 2.12.0 and OCaml 4.10
+##V>=4.10##*)
 
 val rfind : ('a -> bool) -> 'a list -> 'a
 (** [rfind p l] returns the last element [x] of [l] such as [p x] returns
@@ -429,8 +508,13 @@ val filter : ('a -> bool) -> 'a list -> 'a list
     that satisfy the predicate [p].  The order of the elements
     in the input list is preserved.  *)
 
+val count_matching : ('a -> bool) -> 'a list -> int
+(** [count_matching p l] returns the number of elements in [l]
+    that satisfy [p]. Semantically equivalent but faster than
+    [length (filter p l)]. *)
+
 val filteri : (int -> 'a -> bool) -> 'a list -> 'a list
-(** [filter p [a0; a1; ...; an]] returns all the elements [ai] of index [i]
+(** [filteri p [a0; a1; ...; an]] returns all the elements [ai] of index [i]
     that satisfy the predicate [p i ai].  The order of the elements
     in the input list is preserved.
 
@@ -444,7 +528,7 @@ val filter_map : ('a -> 'b option) -> 'a list -> 'b list
     [l] is discarded). *)
 
 val filteri_map : (int -> 'a -> 'b option) -> 'a list -> 'b list
-(** [filter_map f l] calls [(f 0 a0) (f 1 a1).... (f n an)] where [a0,a1..an] are
+(** [filteri_map f l] calls [(f 0 a0) (f 1 a1).... (f n an)] where [a0,a1..an] are
     the elements of [l]. It returns the list of elements [bi] such as
     [f ai = Some bi] (when [f] returns [None], the corresponding element of
     [l] is discarded).
@@ -462,6 +546,19 @@ val partition : ('a -> bool) -> 'a list -> 'a list * 'a list
     elements of [l] that do not satisfy [p].
     The order of the elements in the input list is preserved. *)
 
+val partition_map : ('a -> ('b, 'c) BatEither.t) -> 'a list ->
+  'b list * 'c list
+(** [partition_map f l] returns a pair of lists [(l1, l2)] such that,
+    for each element [x] of the input list [l]:
+    - if [f x] is [Left y1], then [y1] is in [l1], and
+    - if [f x] is [Right y2], then [y2] is in [l2].
+    The output elements are included in [l1] and [l2] in the same
+    relative order as the corresponding input elements in [l].
+    In particular, [partition_map (fun x -> if f x then Left x else Right x) l]
+    is equivalent to [partition f l].
+    @since 3.3.0
+*)
+
 val index_of : 'a -> 'a list -> int option
 (** [index_of e l] returns the index of the first occurrence of [e]
     in [l], or [None] if there is no occurrence of [e] in [l] *)
@@ -526,6 +623,15 @@ val assoc : 'a -> ('a * 'b) list -> 'b
     @raise Not_found if there is no value associated with [a] in the
     list [l]. *)
 
+val assoc_opt: 'a -> ('a * 'b) list -> 'b option
+(** [assoc_opt a l] returns the value associated with key [a] in the list of
+   pairs [l]. That is,
+   [assoc_opt a [ ...; (a,b); ...] = b]
+   if [(a,b)] is the leftmost binding of [a] in list [l].
+   Returns [None] if there is no value associated with [a] in the
+   list [l].
+   @since 2.7.0 *)
+
 val assoc_inv : 'b -> ('a * 'b) list -> 'a
 (** [assoc_inv b l] returns the key associated with value [b] in the list of
     pairs [l]. That is, [assoc b [ ...; (a,b); ...] = a]
@@ -546,6 +652,11 @@ val assq : 'a -> ('a * 'b) list -> 'b
 (** Same as {!List.assoc}, but uses physical equality instead of structural
     equality to compare keys. *)
 
+val assq_opt : 'a -> ('a * 'b) list -> 'b option
+(** Same as {!List.assoc_opt}, but uses physical equality instead of structural
+    equality to compare keys.
+    @since 2.7.0 *)
+
 val assq_inv : 'b -> ('a * 'b) list -> 'a
 (** Same as {!List.assoc_inv}, but uses physical equality instead of structural
     equality to compare keys. *)
@@ -559,8 +670,8 @@ val mem_assq : 'a -> ('a * 'b) list -> bool
     structural equality to compare keys. *)
 
 val modify : 'a -> ('b -> 'b) -> ('a * 'b) list -> ('a * 'b) list
-(** [modify a f l] returns the same list as [l] but with value associated
-    to key [a] replaced with [f a].
+(** [modify a f l] returns the same list as [l] but with value [b] associated
+    to key [a] replaced with [f b].
 
     @raise Not_found if no value is associated with [a] in [l]
     @since 2.1 *)
@@ -587,7 +698,7 @@ val modify_at : int -> ('a -> 'a) -> 'a list -> 'a list
     @since 2.3.0 *)
 
 val modify_opt_at : int -> ('a -> 'a option) -> 'a list -> 'a list
-(** [modify_at_opt n f l] returns the same list as [l] but with
+(** [modify_opt_at n f l] returns the same list as [l] but with
     nth-value [a] removed if [f a] is [None], and replaced by [v] if
     it is [Some v].
 
@@ -642,7 +753,7 @@ val drop : int -> 'a list -> 'a list
     list if [l] have less than [n] elements. *)
 
 val takedrop : int -> 'a list -> 'a list * 'a list
-(** [take_drop n l] is equivalent to [(take n l, drop n l)]
+(** [takedrop n l] is equivalent to [(take n l, drop n l)]
     but is done in one pass.
     @since 2.2.0 *)
 
@@ -664,6 +775,15 @@ val span : ('a -> bool) -> 'a list -> 'a list * 'a list
     @since 2.1
 *)
 
+val fold_while : ('acc -> 'a -> bool) -> ('acc -> 'a -> 'acc) -> 'acc -> 'a list -> 'acc * 'a list
+(** [fold_while p f init l], accumulates elements [x] of list [l] using
+    function [f], as long as predicate [p acc x] holds.
+    At the end, the accumulated value along with the remaining part
+    of the list are returned.
+
+    @since 2.10.0
+*)
+
 val nsplit : ('a -> bool) -> 'a list -> 'a list list
 (** [nsplit], applied to a predicate [p] and a list [xs], returns a
     list of lists. [xs] is split when [p x] is true and [x] is excluded
@@ -698,7 +818,7 @@ val group_consecutive : ('a -> 'a -> bool) -> 'a list -> 'a list list
 
 val interleave : ?first:'a -> ?last:'a -> 'a -> 'a list -> 'a list
 (** [interleave ~first ~last sep [a0;a1;a2;...;an]] returns
-    [first; a0; sep; a1; sep; a2; sep; ...; sep; an; last] *)
+    [first; a0; sep; a1; sep; a2; sep; ...; sep; an; last]. *)
 
 
 (** {6 BatEnum functions}
@@ -709,7 +829,7 @@ val interleave : ?first:'a -> ?last:'a -> 'a -> 'a list -> 'a list
 val enum : 'a list -> 'a BatEnum.t
 (** Returns an enumeration of the elements of a list. This enumeration may
     be used to visit elements of the list in forward order (i.e. from the
-    first element to the last one)*)
+    first element to the last one). *)
 
 val of_enum : 'a BatEnum.t -> 'a list
 (** Build a list from an enumeration. In the result, elements appear in the
@@ -718,7 +838,7 @@ val of_enum : 'a BatEnum.t -> 'a list
 val backwards : 'a list -> 'a BatEnum.t
 (** Returns an enumeration of the elements of a list. This enumeration may
     be used to visit elements of the list in backwards order (i.e. from the
-    last element to the first one)*)
+    last element to the first one). *)
 
 val of_backwards : 'a BatEnum.t -> 'a list
 (** Build a list from an enumeration. The first element of the enumeration
@@ -741,8 +861,8 @@ val combine : 'a list -> 'b list -> ('a * 'b) list
 (** Transform a pair of lists into a list of pairs:
     [combine [a0; a1; ...; an] [b0; b1; ...; bn]] is
     [[(a0,b0); (a1,b1); ...; (an,bn)]].
-    @raise Different_list_size if the two lists
-    have different lengths.  Tail-recursive. *)
+    @raise Invalid_argument if two lists have different lengths.
+    Tail-recursive. *)
 
 
 (** {6 Sorting}*)
@@ -781,7 +901,7 @@ val merge : ('a -> 'a -> int) -> 'a list -> 'a list -> 'a list
 (** Merge two lists:
     Assuming that [l1] and [l2] are sorted according to the
     comparison function [cmp], [merge cmp l1 l2] will return a
-    sorted list containting all the elements of [l1] and [l2].
+    sorted list containing all the elements of [l1] and [l2].
     If several elements compare equal, the elements of [l1] will be
     before the elements of [l2].
     Not tail-recursive (sum of the lengths of the arguments).
@@ -812,7 +932,7 @@ val group : ('a -> 'a -> int) -> 'a list -> 'a list list
 
     For example [group cmp [f;c;b;e;d;a]] can give [[[a;b];[c];[d;e;f]]] if
     following conditions are met:
-    [cmp a b = 0], [cmp b c = -1], [cmp c d = -1], [cmp d e = 0],...
+    [cmp a b = 0], [cmp b c = -1], [cmp c d = -1], [cmp d e = 0], ...
 
     See the note on [group_consecutive].
 *)
@@ -864,6 +984,14 @@ module Comp (T : Comp) : Comp with type t = T.t list
 val nth : 'a list -> int -> 'a
 (** Obsolete. As [at]. *)
 
+val nth_opt: 'a list -> int -> 'a option
+(** Return the [n]-th element of the given list.
+    The first element (head of the list) is at position 0.
+    Return [None] if the list is too short.
+    Raise [Invalid_argument "List.nth"] if [n] is negative.
+    @since 2.7.0
+*)
+
 val takewhile :  ('a -> bool) -> 'a list -> 'a list
 (** obsolete, as {!take_while} *)
 
@@ -876,7 +1004,7 @@ val dropwhile : ('a -> bool) -> 'a list -> 'a list
    The following modules replace functions defined in {!List} with functions
    behaving slightly differently but having the same name. This is by
    design:
-   the functions meant to override the corresponding functions of {!List}.
+   the functions are meant to override the corresponding functions of {!List}.
 
 *)
 
@@ -894,7 +1022,7 @@ module Exceptionless : sig
       that [p x] returns [true] or [None] if such element as not been found. *)
 
   val findi : (int -> 'a -> bool) -> 'a list -> (int * 'a) option
-  (** [findi p e l] returns [Some (i, ai)] where [ai] and [i] are
+  (** [findi p l] returns [Some (i, ai)] where [ai] and [i] are
       respectively the
       first element of [l] and its index, such that [p i ai] is true,
       or [None] if no  such element has been found. *)
@@ -903,7 +1031,7 @@ module Exceptionless : sig
                                     `Invalid_argument of string]
   (** Whenever [n] is inside of [l] size bounds, [split_at n l] returns
       [Ok(l1,l2)], where [l1] contains the first [n] elements of [l] and [l2]
-      contains the others. Otherwise, returns [`Invalid_argument n] *)
+      contains the others. Otherwise, returns [`Invalid_argument n]. *)
 
   val at : 'a list -> int -> [`Ok of 'a | `Invalid_argument of string]
   (** If [n] is inside the bounds of [l], [at l n] returns [Ok x], where
@@ -944,11 +1072,34 @@ module Exceptionless : sig
   val tl : ('a list -> 'a list option)
   (** [tl l] returns [Some x] such that [x] is the given list [l] without its
       first element.
-      Returns [None] if list [l] is empty *)
+      Returns [None] if list [l] is empty. *)
 
   val last : 'a list -> 'a option
     (** [last l] returns either [Some x] where [x] is the last element of the list, or [None] if
         the list is empty. This function takes linear time. *)
+
+  val reduce : ('a -> 'a -> 'a) -> 'a list -> 'a option
+  (** [reduce f h::t] is [Some (fold_left f h t)]
+      and [reduce f []] is None. *)
+
+  val min_max : ?cmp:('a -> 'a -> int) -> 'a list -> ('a * 'a) option
+  (** [min_max l] returns either Some(s, l)
+      where s and l are respectively the smallest and biggest element of [l] as judged by
+      [Pervasives.compare] (by default) or None if [l] is empty.
+	  You can provide another comparison function via the optional [cmp] parameter.
+  *)
+
+  val max : ?cmp:('a -> 'a -> int) -> 'a list -> 'a option
+  (** [max l] returns either [Some x] where [x] is the largest value of the list as judged by
+      [Pervasives.compare] (by default) or [None] is the list is empty.
+	  You can provide another comparison function via the optional [cmp] parameter.
+  *)
+
+  val min : ?cmp:('a -> 'a -> int) -> 'a list -> 'a option
+  (** [min l] returns either [Some x] where [x] is the smallest value of the list as judged by
+      [Pervasives.compare] or [None] is the list is empty.
+	  You can provide another comparison function via the optional [cmp] parameter.
+  *)
 end
 
 (** {6 Infix submodule regrouping all infix operators} *)
@@ -972,6 +1123,7 @@ module Labels : sig
   val mapi : f:(int -> 'a -> 'b) -> 'a list -> 'b list
   val rev_map : f:('a -> 'b) -> 'a list -> 'b list
   val fold_left : f:('a -> 'b -> 'a) -> init:'a -> 'b list -> 'a
+  val fold :      f:('a -> 'b -> 'a) -> init:'a -> 'b list -> 'a
   val fold_right : f:('a -> 'b -> 'b) -> 'a list -> init:'b -> 'b
   val iter2 : f:('a -> 'b -> unit) -> 'a list -> 'b list -> unit
   val map2 : f:('a -> 'b -> 'c) -> 'a list -> 'b list -> 'c list
@@ -985,12 +1137,17 @@ module Labels : sig
   val subset : cmp:('a -> 'b -> int) -> 'a list -> 'b list -> bool
   val find : f:('a -> bool) -> 'a list -> 'a
   val find_exn : f:('a -> bool) -> exn -> 'a list -> 'a
+##V>=4.10##val find_map_opt: f:('a -> 'b option) -> 'a list -> 'b option
   val findi : f:(int -> 'a -> bool) -> 'a list -> (int * 'a)
   val rfind : f:('a -> bool) -> 'a list -> 'a
   val filter : f:('a -> bool) -> 'a list -> 'a list
   val filter_map : f:('a -> 'b option) -> 'a list -> 'b list
+  val count_matching : f:('a -> bool) -> 'a list -> int
+##V>=4.10##val concat_map : f:('a -> 'b list) -> 'a list -> 'b list
   val find_all : f:('a -> bool) -> 'a list -> 'a list
   val partition : f:('a -> bool) -> 'a list -> 'a list * 'a list
+  val partition_map : f:('a -> ('b, 'c) BatEither.t) -> 'a list ->
+    'b list * 'c list
   val remove_if : f:('a -> bool) -> 'a list -> 'a list
   val take_while : f:('a -> bool) -> 'a list -> 'a list
   val drop_while : f:('a -> bool) -> 'a list -> 'a list
@@ -1010,6 +1167,8 @@ module Labels : sig
   end
 end
 
-
 val ( @ ) : 'a list -> 'a list -> 'a list
   (** Tail recursive [List.append]. *)
+
+##V>=5.1##val find_index : ('a -> bool) -> 'a list -> int option
+##V>=5.1##val find_mapi : (int -> 'a -> 'b option) -> 'a list -> 'b option
