@@ -90,7 +90,7 @@
 (** {6 Scanning buffers} *)
 module Scanning : sig
 
-  type scanbuf = Scanf.Scanning.scanbuf;;
+  type scanbuf = Scanf.Scanning.scanbuf
   (** The type of scanning buffers. A scanning buffer is the source from which a
       formatted input function gets characters. The scanning buffer holds the
       current state of the scan, plus a function to get the next char from the
@@ -101,7 +101,7 @@ module Scanning : sig
       it is stored back in the scanning buffer and becomes the next
       character read. *)
 
-  val stdib : scanbuf;;
+  val stdib : scanbuf
   (** The scanning buffer reading from [stdin].
       [stdib] is equivalent to [Scanning.from_input stdin].
 
@@ -110,13 +110,13 @@ module Scanning : sig
       specifications must properly skip this character (simply add a ['\n']
       as the last character of the format string). *)
 
-  val from_string : string -> scanbuf;;
+  val from_string : string -> scanbuf
   (** [Scanning.from_string s] returns a scanning buffer which reads from the
       given string.
       Reading starts from the first character in the string.
       The end-of-input condition is set when the end of the string is reached. *)
 
-  val from_file : string -> scanbuf;;
+  val from_file : string -> scanbuf
   (** Bufferized file reading in text mode. The efficient and usual
       way to scan text mode files (in effect, [from_file] returns a
       scanning buffer that reads characters in large chunks, rather than one
@@ -124,10 +124,10 @@ module Scanning : sig
       [Scanning.from_file fname] returns a scanning buffer which reads
       from the given file [fname] in text mode. *)
 
-  val from_file_bin : string -> scanbuf;;
+  val from_file_bin : string -> scanbuf
   (** Bufferized file reading in binary mode. *)
 
-  val from_function : (unit -> char) -> scanbuf;;
+  val from_function : (unit -> char) -> scanbuf
   (** [Scanning.from_function f] returns a scanning buffer with the given
       function as its reading method.
 
@@ -136,19 +136,19 @@ module Scanning : sig
       When the function has no more character to provide, it must signal an
       end-of-input condition by raising the exception [End_of_file]. *)
 
-  val from_input : BatIO.input -> scanbuf;;
+  val from_input : BatIO.input -> scanbuf
   (** [Scanning.from_input ic] returns a scanning buffer which reads from the
       input channel [ic], starting at the current reading position. *)
 
-  val end_of_input : scanbuf -> bool;;
+  val end_of_input : scanbuf -> bool
   (** [Scanning.end_of_input ib] tests the end-of-input condition of the given
       scanning buffer. *)
 
-  val beginning_of_input : scanbuf -> bool;;
+  val beginning_of_input : scanbuf -> bool
   (** [Scanning.beginning_of_input ib] tests the beginning of input condition of
       the given scanning buffer. *)
 
-  val name_of_input : scanbuf -> string;;
+  val name_of_input : scanbuf -> string
   (** [Scanning.name_of_input ib] returns the name of the character source
       for the scanning buffer [ib]. *)
 
@@ -156,16 +156,16 @@ module Scanning : sig
      {6 Obsolete}
   *)
 
-  val from_channel : BatIO.input -> scanbuf;;
+  val from_channel : BatIO.input -> scanbuf
   (** @obsolete use {!from_input}*)
 
 
-end;;
+end
 
 (** {6 Type of formatted input functions} *)
 
 type ('a, 'b, 'c, 'd) scanner =
-  ('a, Scanning.scanbuf, 'b, 'c, 'a -> 'd, 'd) format6 -> 'c;;
+  ('a, Scanning.scanbuf, 'b, 'c, 'a -> 'd, 'd) format6 -> 'c
 (** The type of formatted input scanners: [('a, 'b, 'c, 'd) scanner] is the
     type of a formatted input function that reads from some scanning buffer
     according to some format string; more precisely, if [scan] is some
@@ -184,13 +184,13 @@ type ('a, 'b, 'c, 'd) scanner =
     then [bscanf ib "%r;" read_elem f] reads a value [v] of type [t] followed
     by a [';'] character, and returns [f v]. *)
 
-exception Scan_failure of string;;
+exception Scan_failure of string
 (** The exception that formatted input functions raise when the input cannot be
     read according to the given format. *)
 
 (** {6 The general formatted input function} *)
 
-val bscanf : Scanning.scanbuf -> ('a, 'b, 'c, 'd) scanner;;
+val bscanf : Scanning.scanbuf -> ('a, 'b, 'c, 'd) scanner
 (** [bscanf ib fmt r1 ... rN f] reads arguments for the function [f], from the
     scanning buffer [ib], according to the format string [fmt], and applies [f]
     to these values.
@@ -381,29 +381,29 @@ val bscanf : Scanning.scanbuf -> ('a, 'b, 'c, 'd) scanner;;
 
 (** {6 Specialized formatted input functions} *)
 
-val fscanf : in_channel -> ('a, 'b, 'c, 'd) scanner;;
-(** Same as {!Scanf.bscanf}, but reads from the given channel.
+##V<5##val fscanf : in_channel -> ('a, 'b, 'c, 'd) scanner
+##V<5##(** Same as {!Scanf.bscanf}, but reads from the given channel.
+##V<5##
+##V<5##    Warning: since all formatted input functions operate from a scanning
+##V<5##    buffer, be aware that each [fscanf] invocation will operate with a
+##V<5##    scanning buffer reading from the given channel. This extra level of
+##V<5##    bufferization can lead to strange scanning behaviour if you use low level
+##V<5##    primitives on the channel (reading characters, seeking the reading
+##V<5##    position, and so on).
+##V<5##
+##V<5##    As a consequence, never mixt direct low level reading and high level
+##V<5##    scanning from the same input channel. *)
 
-    Warning: since all formatted input functions operate from a scanning
-    buffer, be aware that each [fscanf] invocation will operate with a
-    scanning buffer reading from the given channel. This extra level of
-    bufferization can lead to strange scanning behaviour if you use low level
-    primitives on the channel (reading characters, seeking the reading
-    position, and so on).
-
-    As a consequence, never mixt direct low level reading and high level
-    scanning from the same input channel. *)
-
-val sscanf : string -> ('a, 'b, 'c, 'd) scanner;;
+val sscanf : string -> ('a, 'b, 'c, 'd) scanner
 (** Same as {!Scanf.bscanf}, but reads from the given string. *)
 
-val scanf : ('a, 'b, 'c, 'd) scanner;;
+val scanf : ('a, 'b, 'c, 'd) scanner
 (** Same as {!Scanf.bscanf}, but reads from the predefined scanning
     buffer {!Scanf.Scanning.stdib} that is connected to [stdin]. *)
 
 val kscanf :
   Scanning.scanbuf -> (Scanning.scanbuf -> exn -> 'd) ->
-  ('a, 'b, 'c, 'd) scanner;;
+  ('a, 'b, 'c, 'd) scanner
 (** Same as {!Scanf.bscanf}, but takes an additional function argument
     [ef] that is called in case of error: if the scanning process or
     some conversion fails, the scanning function aborts and calls the
@@ -414,7 +414,7 @@ val kscanf :
 
 val bscanf_format :
   Scanning.scanbuf -> ('a, 'b, 'c, 'd, 'e, 'f) format6 ->
-  (('a, 'b, 'c, 'd, 'e, 'f) format6 -> 'g) -> 'g;;
+  (('a, 'b, 'c, 'd, 'e, 'f) format6 -> 'g) -> 'g
 (** [bscanf_format ib fmt f] reads a format string token from the scanning
     buffer [ib], according to the given format string [fmt], and applies [f] to
     the resulting format string value.
@@ -423,12 +423,12 @@ val bscanf_format :
 
 val sscanf_format :
   string -> ('a, 'b, 'c, 'd, 'e, 'f) format6 ->
-  (('a, 'b, 'c, 'd, 'e, 'f) format6 -> 'g) -> 'g;;
+  (('a, 'b, 'c, 'd, 'e, 'f) format6 -> 'g) -> 'g
 (** Same as {!Scanf.bscanf_format}, but reads from the given string. *)
 
 val format_from_string :
   string ->
-  ('a, 'b, 'c, 'd, 'e, 'f) format6 -> ('a, 'b, 'c, 'd, 'e, 'f) format6;;
+  ('a, 'b, 'c, 'd, 'e, 'f) format6 -> ('a, 'b, 'c, 'd, 'e, 'f) format6
 (** [format_from_string s fmt] converts a string argument to a format string,
     according to the given format string [fmt].
     @raise Scan_failure if [s], considered as a format string, does not
